/*
 *  Miscellaneous functions used by the cdecl99 test suite.
 *  Copyright © 2011-2012, 2021-2023 Nick Bowler
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <getopt.h>
#include <cdecl.h>

#include "help.h"
#include "test.h"
#include "intconv.h"

void *realloc_nofail(void *ptr, size_t size)
{
	void *p;
	
	p = realloc(ptr, size);
	if (!p) {
		perror("failed to allocate memory");
		abort();
	}

	return p;
}

void *malloc_nofail(size_t size)
{
	return realloc_nofail(NULL, size);
}

static unsigned intconv_base(const char **str)
{
	if ((*str)[0] == '0') {
		if ((*str)[1] == 'X' || (*str)[1] == 'x') {
			*str += 2;
			return INTCONV_HEXADECIMAL;
		}

		return INTCONV_OCTAL;
	}

	return INTCONV_DECIMAL;
}

bool test_strtoumax(uintmax_t *out, const char *s, uintmax_t limit)
{
	static const char idx[] = "0123456789abcdef0123456789ABCDEF";
	unsigned base = intconv_base(&s);
	uintmax_t v;
	char *c, d;

	for (v = 0; (d = *s++);) {
		if (!(c = strchr(idx, d)) || (d = (c-idx) & 0xf) >= base)
			return 0;

		if (!intconv_shift(&v, base, d) || v > limit)
			return 0;
	}

	*out = v;
	return true;
}

void test_print_options(const struct option *lopts)
{
	const struct option *opt;

	puts("Options:");
	for (opt = lopts; opt->val; opt++) {
		if (help_print_optstring(opt, "ARG", 20))
			putchar('\n');
	}
}
